package com.jiuqi.grid;

import com.jiuqi.util.Base64;
import com.jiuqi.util.DateUtil;
import com.jiuqi.util.JqLib;
import com.jiuqi.util.MemStream;
import com.jiuqi.util.Stream;
import com.jiuqi.util.StreamException;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.util.Date;

public class GridCell implements Serializable {
   private int col;
   private int row;
   private byte[] data = new byte[20];
   protected GridData grid;

   protected void dataChanged() {
   }

   public static int BGR2RGB(int color) {
      if (color < 0) {
         return color;
      } else {
         int red = (color & 255) << 16;
         int blue = (color & 16711680) >> 16;
         int green = color & '\uff00';
         return red | blue | green;
      }
   }

   public static int RGB2BGR(int color) {
      if (color < 0) {
         return color;
      } else {
         int red = (color & 16711680) >> 16;
         int blue = (color & 255) << 16;
         int green = color & '\uff00';
         return red | blue | green;
      }
   }

   private boolean getBit(int bytePos, int bitPos) {
      return (this.data[bytePos] >> bitPos & 1) > 0;
   }

   private void setBit(int bytePos, int bitPos, boolean value) {
      byte flag = (byte)(1 << bitPos);
      if (value) {
         this.data[bytePos] |= flag;
      } else {
         this.data[bytePos] = (byte)(this.data[bytePos] & ~flag);
      }

   }

   private static int byteToInt(byte b) {
      return b < 0 ? 127 + Math.abs(b) : b;
   }

   public final void init(GridData g, int aCol, int aRow) {
      this.init(g, aCol, aRow, g.getCellPropData(aCol, aRow));
   }

   public final void init(GridData g, int aCol, int aRow, byte[] celldata) {
      System.arraycopy(celldata, 0, this.data, 0, celldata.length);
      this.grid = g;
      this.col = aCol;
      this.row = aRow;
   }

   public final byte[] getPropData() {
      return this.data;
   }

   public final int getBackColor() {
      return BGR2RGB(this.grid.cellColors().get(byteToInt(this.data[0])));
   }

   public final boolean getBackLines(int cellBackStyle) {
      return cellBackStyle >= 0 && cellBackStyle <= 3 ? this.getBit(1, cellBackStyle + 4) : false;
   }

   public final int getBackStyle() {
      return this.data[2] >> 0 & 63;
   }

   public final int getBEdgeColor() {
      return BGR2RGB(this.grid.edgeColors().get(this.data[10] >> 4 & 15));
   }

   public final int getBEdgeStyle() {
      return this.data[10] >> 0 & 15;
   }

   public final boolean getCanInput() {
      return this.getBit(2, 7);
   }

   public final boolean getCanModify() {
      return this.getBit(2, 6);
   }

   public final boolean getCanRead() {
      return this.getBit(4, 7);
   }

   public final boolean getCanSelect() {
      return this.getBit(4, 6);
   }

   public final boolean getCanWrite() {
      return this.getBit(5, 7);
   }

   public final int getDataType() {
      return this.data[3] >> 4 & 15;
   }

   public final boolean getEditJumpNext() {
      return this.getBit(5, 6);
   }

   public final String getEditMask() {
      return null;
   }

   public final int getEditMode() {
      return this.data[3] >> 0 & 15;
   }

   public final boolean getEmptyCell() {
      return this.getBit(6, 7);
   }

   public final boolean getFitFontSize() {
      return this.getBit(6, 6);
   }

   public final boolean getFontBold() {
      return this.getBit(7, 7);
   }

   public final int getFontColor() {
      int index = this.data[6] >> 0 & 63;
      return BGR2RGB(this.grid.fontColors().get(index));
   }

   public final int getFontHeight() {
      return this.getFontSize() * 96 / 72;
   }

   public final boolean getFontItalic() {
      return this.getBit(7, 6);
   }

   public final String getFontName() {
      int index = this.data[4] & 63;
      return this.grid.fontNames().getFont(index);
   }

   public final int getFontSize() {
      int index = this.data[5] & 63;
      int h = this.grid.fontSizes().get(index);
      return h;
   }

   public final boolean getFontStrikeOut() {
      return this.getBit(7, 5);
   }

   public final boolean getFontStyle(int fontStyle) {
      return fontStyle >= 0 && fontStyle < 4 ? this.getBit(1, fontStyle) : false;
   }

   public final boolean getFontUnderLine() {
      return this.getBit(7, 4);
   }

   public final int getHorzAlign() {
      return this.data[7] >> 0 & 7;
   }

   public final int getImeMode() {
      return this.data[11] >> 0 & 7;
   }

   public final int getIndent() {
      return this.data[1] >> 0 & 15;
   }

   public final int getLEdgeColor() {
      return this.col == 0 ? 0 : this.grid.getCellEx(this.col - 1, this.row).getREdgeColor();
   }

   public final int getLEdgeStyle() {
      return this.col == 0 ? 0 : this.grid.getCellEx(this.col - 1, this.row).getREdgeStyle();
   }

   public final boolean getMultiLine() {
      return this.getBit(8, 7);
   }

   public final boolean getReadOnly() {
      return !this.getCanModify() || !this.getCanWrite();
   }

   public final int getREdgeColor() {
      int index = this.data[9] >> 4 & 15;
      return BGR2RGB(this.grid.edgeColors().get(index));
   }

   public final int getREdgeStyle() {
      return this.data[9] >> 0 & 15;
   }

   public final String getShowFormat() {
      return null;
   }

   public final boolean getSilverHead() {
      return this.getBit(8, 6);
   }

   public final int getTEdgeColor() {
      return this.row == 0 ? 0 : this.grid.getCellEx(this.col, this.row - 1).getBEdgeColor();
   }

   public final int getTEdgeStyle() {
      return this.row == 0 ? 0 : this.grid.getCellEx(this.col, this.row - 1).getBEdgeStyle();
   }

   public final int getVertAlign() {
      return this.data[8] >> 0 & 7;
   }

   public final boolean getVertText() {
      return this.getBit(8, 5);
   }

   public final boolean getWrapLine() {
      return this.getBit(8, 4);
   }

   public final boolean getBoolean() {
      return JqLib.isBoolTrue(this.grid.getCellData(this.col, this.row));
   }

   public final Date getDate() {
      return DateUtil.tryParserDateString(this.grid.getCellData(this.col, this.row));
   }

   public final Date getDateTime() {
      return this.getDate();
   }

   public final double getFloat() {
      String strData = this.grid.getCellData(this.col, this.row);
      if (strData == null) {
         return 0.0;
      } else {
         StringBuffer buffer = new StringBuffer();

         for(int i = 0; i < strData.length(); ++i) {
            char ch = strData.charAt(i);
            if (Character.isDigit(ch) || ch == '.' || ch == '-' || ch == 'E' || ch == 'e') {
               buffer.append(ch);
            }
         }

         if (buffer.length() == 0) {
            return 0.0;
         } else {
            return Double.parseDouble(buffer.toString());
         }
      }
   }

   public final int getInteger() {
      String strData = this.grid.getCellData(this.col, this.row);
      if (strData == null) {
         return 0;
      } else {
         StringBuffer buffer = new StringBuffer();

         for(int i = 0; i < strData.length(); ++i) {
            char ch = strData.charAt(i);
            if (!Character.isDigit(ch) && ch != '-') {
               if (ch == '.') {
                  break;
               }
            } else {
               buffer.append(ch);
            }
         }

         return buffer.length() == 0 ? 0 : Integer.parseInt(buffer.toString());
      }
   }

   public final String getString() {
      return this.grid.getCellData(this.col, this.row);
   }

   public final double getTime() {
      return this.getFloat();
   }

   public final int getCellBottom() {
      CellField cf = this.grid.expandCell(this.col, this.row);
      return cf.bottom;
   }

   public final String getCellData() {
      return this.grid.getCellData(this.col, this.row);
   }

   public final CellField getCellField() {
      return this.grid.expandCell(this.col, this.row);
   }

   public final int getCellHeight() {
      CellField cf = this.grid.expandCell(this.col, this.row);
      int rt = 0;

      for(int i = cf.top; i <= cf.bottom; ++i) {
         rt += this.grid.getRowHeights(i);
      }

      return rt;
   }

   public final int getCellLeft() {
      return this.grid.expandCell(this.col, this.row).left;
   }

   public final int getCellRight() {
      return this.grid.expandCell(this.col, this.row).right;
   }

   public final int getCellTop() {
      return this.grid.expandCell(this.col, this.row).top;
   }

   public final int getCellWidth() {
      CellField cf = this.grid.expandCell(this.col, this.row);
      int rt = 0;

      for(int i = cf.left; i <= cf.right; ++i) {
         rt += this.grid.getColWidths(i);
      }

      return rt;
   }

   public final int getColNum() {
      return this.col;
   }

   public final int getColSpan() {
      CellField cf = this.grid.expandCell(this.col, this.row);
      return cf.left == this.col && cf.top == this.row ? cf.right - cf.left + 1 : 0;
   }

   public final int getColWidth() {
      return this.grid.getColWidths(this.col);
   }

   public final boolean getHasConjection() {
      CellField cf = this.grid.expandCell(this.col, this.row);
      return cf.left != cf.right || cf.top != cf.bottom;
   }

   public final boolean getIsNull() {
      return this.grid.getCellData(this.col, this.row) == null;
   }

   public final boolean getIsZero() {
      return this.getIsNull() | this.getInteger() == 0;
   }

   public final int getRowHeight() {
      return this.grid.getRowHeights(this.row);
   }

   public final int getRowNum() {
      return this.row;
   }

   public final int getRowSpan() {
      CellField cf = this.grid.expandCell(this.col, this.row);
      return cf.left == this.col && cf.top == this.row ? cf.bottom - cf.top + 1 : 0;
   }

   public final boolean getVisible() {
      if (this.grid.getColVisible(this.col) && this.grid.getRowVisible(this.row)) {
         CellField cf = this.grid.expandCell(this.col, this.row);
         return cf.left == this.col && cf.top == this.row;
      } else {
         return false;
      }
   }

   public final int getOutputKind() {
      return this.data[11] >> 4 & 3;
   }

   public final void setBackColor(int value) {
      int color = RGB2BGR(value);
      int index = this.grid.cellColors().indexOf(color);
      if (index < 0) {
         if (this.grid.cellColors().count() < 127) {
            index = this.grid.cellColors().add(color);
         } else {
            index = 0;
         }
      }

      this.data[0] = (byte)index;
      this.dataChanged();
   }

   public final void setBackLines(int cellBackStyle, boolean value) {
      if (cellBackStyle >= 0 && cellBackStyle <= 3) {
         this.setBit(1, cellBackStyle + 4, value);
      }

      this.dataChanged();
   }

   public final void setBackStyle(int value) {
      this.data[2] = (byte)(this.data[2] & 192 | (byte)value & 63);
      if (value < 28) {
         this.setBit(8, 6, false);
      }

      this.dataChanged();
   }

   public final void setBEdgeColor(int value) {
      int color = RGB2BGR(value);
      int index = this.grid.edgeColors().indexOf(color);
      if (index < 0) {
         if (this.grid.edgeColors().count() < 15) {
            index = this.grid.edgeColors().add(color);
         } else {
            index = 0;
         }
      }

      this.data[10] = (byte)(this.data[10] & 15 | index << 4 & 240);
      this.dataChanged();
   }

   public final void setBEdgeStyle(int value) {
      this.data[10] = (byte)(this.data[10] & 240 | value & 15);
      this.dataChanged();
   }

   public final void setCanInput(boolean value) {
      this.setBit(2, 7, value);
      this.dataChanged();
   }

   public final void setCanModify(boolean value) {
      this.setBit(2, 6, value);
      this.dataChanged();
   }

   public final void setCanRead(boolean value) {
      this.setBit(4, 7, value);
      this.dataChanged();
   }

   public final void setCanSelect(boolean value) {
      this.setBit(4, 6, value);
      this.dataChanged();
   }

   public final void setCanWrite(boolean value) {
      this.setBit(5, 7, value);
      this.dataChanged();
   }

   public final void setDataType(int value) {
      this.data[3] = (byte)(this.data[3] & 15 | value << 4 & 240);
      this.dataChanged();
   }

   public final void setEditJumpNext(boolean value) {
      this.setBit(5, 6, value);
      this.dataChanged();
   }

   public final void setEditMask(String value) {
   }

   public final void setEditMode(int value) {
      this.data[3] = (byte)(this.data[3] & 240 | value & 15);
      this.dataChanged();
   }

   public final void setEmptyCell(boolean value) {
      this.setBit(6, 7, value);
      this.dataChanged();
   }

   public final void setFitFontSize(boolean value) {
      this.setBit(6, 6, value);
      this.dataChanged();
   }

   public final void setFontBold(boolean value) {
      this.setBit(7, 7, value);
      this.dataChanged();
   }

   public final void setFontColor(int value) {
      int color = RGB2BGR(value);
      int index = this.grid.fontColors().indexOf(color);
      if (index < 0) {
         if (this.grid.fontColors().count() < 63) {
            index = this.grid.fontColors().add(color);
         } else {
            index = 63;
         }
      }

      this.data[6] = (byte)(this.data[6] & 192 | index & 63);
      this.dataChanged();
   }

   public final void setFontHeight(int value) {
      this.setFontSize(value * 72 / 96);
   }

   public final void setFontItalic(boolean value) {
      this.setBit(7, 6, value);
      this.dataChanged();
   }

   public final void setFontName(String value) {
      int index = this.grid.fontNames().indexOfFont(value);
      if (index == -1) {
         if (this.grid.fontNames().count() < 63) {
            index = this.grid.fontNames().addFont(value);
         } else {
            index = 0;
         }
      }

      this.data[4] = (byte)(this.data[4] & 192 | index & 63);
      this.dataChanged();
   }

   public final void setFontSize(int value) {
      int fontsize = value;
      int index = this.grid.fontSizes().indexOf(fontsize);
      if (index < 0) {
         if (this.grid.fontSizes().count() < 63) {
            index = this.grid.fontSizes().add(fontsize);
         } else {
            index = 0;
         }
      }

      this.data[5] = (byte)(this.data[5] & 192 | index & 63);
      this.dataChanged();
   }

   public final void setFontStrikeOut(boolean value) {
      this.setBit(7, 5, value);
      this.dataChanged();
   }

   public final void setFontStyle(int fontStyle, boolean value) {
      if (fontStyle >= 0 && fontStyle < 4) {
         this.setBit(1, fontStyle, value);
      }

      this.dataChanged();
   }

   public final void setFontUnderLine(boolean value) {
      this.setBit(7, 4, value);
      this.dataChanged();
   }

   public final void setHorzAlign(int value) {
      this.data[7] = (byte)(this.data[7] & 248 | value & 7);
      this.dataChanged();
   }

   public final void setImeMode(int value) {
      this.data[11] = (byte)(this.data[11] & 248 | value & 7);
      this.dataChanged();
   }

   public final void setIndent(int value) {
      this.data[1] = (byte)(this.data[1] & 240 | value & 15);
      this.dataChanged();
   }

   public void setLEdgeColor(int value) {
      if (this.col != 0) {
         GridCell cell = this.grid.getCellEx(this.col - 1, this.row);
         cell.setREdgeColor(value);
         this.grid.setCell(this.col - 1, this.row, cell);
      }
   }

   public void setLEdgeStyle(int value) {
      if (this.col != 0) {
         GridCell cell = this.grid.getCellEx(this.col - 1, this.row);
         cell.setREdgeStyle(value);
         this.grid.setCell(this.col - 1, this.row, cell);
      }
   }

   public final void setMultiLine(boolean value) {
      this.setBit(8, 7, value);
      this.dataChanged();
   }

   public final void setREdgeColor(int value) {
      int color = RGB2BGR(value);
      int index = this.grid.edgeColors().indexOf(color);
      if (index < 0) {
         if (this.grid.edgeColors().count() < 15) {
            index = this.grid.edgeColors().add(color);
         } else {
            index = 0;
         }
      }

      this.data[9] = (byte)(this.data[9] & 15 | index << 4 & 240);
      this.dataChanged();
   }

   public final void setREdgeStyle(int value) {
      this.data[9] = (byte)(this.data[9] & 240 | value & 15);
      this.dataChanged();
   }

   public final void setShowFormat(String value) {
   }

   public final void setSilverHead(boolean value) {
      if (this.getBit(8, 6) != value) {
         this.setBit(8, 6, value);
         if (value) {
            this.setBackColor(-16777201);
            this.setBackStyle(28);
            this.setLEdgeColor(8421504);
            this.setREdgeColor(8421504);
            this.setTEdgeColor(8421504);
            this.setBEdgeColor(8421504);
         } else {
            this.setBackColor(-16777211);
            this.setBackStyle(0);
            this.setLEdgeColor(-16777201);
            this.setREdgeColor(-16777201);
            this.setTEdgeColor(-16777201);
            this.setBEdgeColor(-16777201);
         }

         this.dataChanged();
      }
   }

   public void setTEdgeColor(int value) {
      if (this.row != 0) {
         GridCell cell = this.grid.getCellEx(this.col, this.row - 1);
         cell.setBEdgeColor(value);
         this.grid.setCell(this.col, this.row - 1, cell);
      }
   }

   public void setTEdgeStyle(int value) {
      if (this.row != 0) {
         GridCell cell = this.grid.getCellEx(this.col, this.row - 1);
         cell.setBEdgeStyle(value);
         this.grid.setCell(this.col, this.row - 1, cell);
      }
   }

   public final void setVertAlign(int value) {
      this.data[8] = (byte)(this.data[8] & 248 | value & 7);
      this.dataChanged();
   }

   public final void setVertText(boolean value) {
      this.setBit(8, 5, value);
      this.dataChanged();
   }

   public final void setWrapLine(boolean value) {
      this.setBit(8, 4, value);
      this.dataChanged();
   }

   public final void setBoolean(boolean value) {
      this.grid.setCellData(this.col, this.row, String.valueOf(value));
   }

   public final void setDate(Date value) {
      String str = value == null ? null : DateUtil.convertDateToString1(value);
      this.grid.setCellData(this.col, this.row, str);
   }

   public final void setDateTime(Date value) {
      this.setDate(value);
   }

   public final void setFloat(double value) {
      this.grid.setCellData(this.col, this.row, String.valueOf(value));
   }

   public final void setInteger(int value) {
      this.grid.setCellData(this.col, this.row, String.valueOf(value));
   }

   public final void setString(String value) {
      this.grid.setCellData(this.col, this.row, value);
   }

   public final void setTime(double value) {
      this.grid.setCellData(this.col, this.row, String.valueOf(value));
   }

   public final void setCellData(String value) {
      this.grid.setCellData(this.col, this.row, value);
   }

   public final void setColNum(int value) {
      this.col = value;
   }

   public final void setColWidth(int value) {
      this.grid.setColWidths(this.col, value);
   }

   /** @deprecated */
   public final void setHasConjection(boolean value) {
   }

   public final void setIsNull(boolean value) {
      this.grid.setCellData(this.col, this.row, value ? null : "");
   }

   public final void setRowHeight(int value) {
      this.grid.setRowHeights(this.row, value);
   }

   public final void setRowNum(int value) {
      this.row = value;
   }

   /** @deprecated */
   public final void setVisible(boolean value) {
   }

   public final String getCssClass() {
      return this.grid.getCellClass(this.col, this.row);
   }

   public final void setCssClass(String value) {
      this.grid.setCellClass(this.col, this.row, value);
   }

   public final String getImageReference() {
      String result = this.getCellData();
      if (result != null && result.indexOf(58) > 0) {
         String ext = result.substring(0, result.indexOf(58));
         ext = ext.toUpperCase();
         if (ext.equals("JPG") || ext.equals("BMP") || ext.equals("GIF") || ext.equals("WMF") || ext.equals("PNG") || ext.equals("JPEG")) {
            result = null;
         }
      }

      return result;
   }

   public final void setImageReference(String ref) {
      this.setDataType(6);
      this.setCellData(ref);
   }

   public final String getImageType() {
      String data = this.getCellData();
      if (data == null) {
         return null;
      } else {
         int pos = data.indexOf(58);
         if (pos >= 0) {
            String ext = data.substring(0, pos);
            return ext.toUpperCase();
         } else {
            return null;
         }
      }
   }

   public final byte[] getImageData() {
      String data = this.getCellData();
      if (data == null) {
         return null;
      } else {
         int pos = data.indexOf(58);
         if (pos >= 0) {
            String imgStr = data.substring(pos + 1);
            return Base64.base64ToByteArray(imgStr);
         } else {
            return null;
         }
      }
   }

   public final void setImageData(byte[] data, String imgExtension) {
      String ext = imgExtension != null && imgExtension.length() != 0 ? imgExtension.toUpperCase() : "JPG";
      String cellData = ext + ":" + Base64.byteArrayToBase64(data);
      this.setDataType(6);
      this.setCellData(cellData);
   }

   public final String[] getLinkInformation() {
      return parserLinkInformation(this.getCellData());
   }

   public static final String[] parserLinkInformation(String linkValue) {
      String[] linkInfo = new String[3];
      if (linkValue != null) {
         if (linkValue.length() <= 5) {
            linkInfo[0] = linkValue;
         } else {
            String head = linkValue.substring(0, 5).toUpperCase();
            if (!head.equals("LINK:")) {
               linkInfo[0] = linkValue;
            } else {
               try {
                  byte[] data = Base64.base64ToByteArray(linkValue.substring(5));
                  MemStream stream = new MemStream();
                  stream.writeBuffer(data, 0, data.length);
                  stream.setPosition(0L);
                  int count = stream.readInt();
                  linkInfo[0] = stream.readString(count);
                  count = stream.readInt();
                  linkInfo[1] = stream.readString(count);
                  count = stream.readInt();
                  linkInfo[2] = stream.readString(count);
               } catch (Exception var6) {
                  Exception ex = var6;
                  linkInfo[0] = linkValue;
                  linkInfo[1] = null;
                  linkInfo[2] = null;
                  ex.printStackTrace();
               }
            }
         }
      }

      return linkInfo;
   }

   public final void setLinkInformation(String text, String reference, String target) {
      this.setDataType(9);
      String data = makeLinkInformation(text, reference, target);
      this.setCellData(data);
   }

   private static void writeString(Stream stream, String text) throws StreamException {
      if (text != null && text.length() != 0) {
         byte[] data;
         try {
            data = text.getBytes(stream.getCharset());
         } catch (UnsupportedEncodingException var3) {
            data = text.getBytes();
         }

         stream.writeInt(data.length);
         stream.writeBuffer(data, 0, data.length);
      } else {
         stream.writeInt(0);
      }

   }

   public static final String makeLinkInformation(String text, String reference, String target) {
      try {
         MemStream stream = new MemStream();
         writeString(stream, text);
         writeString(stream, reference);
         writeString(stream, target);
         String data = Base64.byteArrayToBase64(stream.getBytes());
         return "LINK:" + data;
      } catch (StreamException var5) {
         StreamException e = var5;
         e.printStackTrace();
         return null;
      }
   }

   public final int getDataFlag() {
      return this.data[12] & 15;
   }

   public final void setDataFlag(int value) {
      this.data[12] = (byte)(this.data[12] & 240 | value & 15);
      this.dataChanged();
   }

   public final int getDataFormat() {
      return this.data[12] >>> 4;
   }

   public final void setDataFormat(int value) {
      this.data[12] = (byte)(this.data[12] & 15 | (value & 15) << 4);
      this.dataChanged();
   }

   public final int getDataProperty() {
      int r = this.data[14];
      r = (r << 8) + this.data[13];
      return r;
   }

   public final void setDataProperty(int value) {
      this.data[14] = (byte)(value >>> 8);
      this.data[13] = (byte)(value & 255);
      this.dataChanged();
   }

   public final void setOutputKind(int value) {
      this.data[11] = (byte)(this.data[11] & 207 | value << 4 & 48);
      this.dataChanged();
   }

   public final String getScript() {
      return this.grid.getCellScript(this.col, this.row);
   }

   public final void setScript(String value) {
      this.grid.setCellScript(this.col, this.row, value);
   }

   public final int getWidth() {
      int width = 0;
      CellField cf = this.grid.expandCell(this.col, this.row);

      for(int i = cf.left; i <= cf.right; ++i) {
         width += this.grid.getColWidths(i);
      }

      return width;
   }

   public final int getHeight() {
      int height = 0;
      CellField cf = this.grid.expandCell(this.col, this.row);

      for(int i = cf.top; i <= cf.bottom; ++i) {
         height += this.grid.getRowHeights(i);
      }

      return height;
   }

   public String toString() {
      StringBuffer result = new StringBuffer();
      result.append("[");
      result.append(this.col);
      result.append(",");
      result.append(this.row);
      result.append("]:");
      result.append(this.getCellData());
      return result.toString();
   }

   public final String getEditText() {
      String storeText = this.getCellData();
      if (storeText == null) {
         return null;
      } else {
         int dataType = this.getDataType();
         CellDataProperty cdp = new CellDataProperty(this.getDataType(), this.getDataFlag(), this.getEditMode(), this.getDataFormat(), this.getDataProperty());
         switch (dataType) {
            case 1:
               return (new TextCellProperty(cdp)).getEditText(storeText);
            case 2:
               return (new NumberCellProperty(cdp)).getEditText(storeText);
            case 3:
               return (new CurrencyCellProperty(cdp)).getEditText(storeText);
            default:
               return storeText;
         }
      }
   }

   public final void setEditText(String editText) {
      this.setCellData(this.parseEditText(editText));
   }

   public final String getShowText() {
      String storeText = this.getCellData();
      if (storeText == null) {
         return null;
      } else {
         int dataType = this.getDataType();
         CellDataProperty cdp = new CellDataProperty(this.getDataType(), this.getDataFlag(), this.getEditMode(), this.getDataFormat(), this.getDataProperty());
         switch (dataType) {
            case 1:
               return (new TextCellProperty(cdp)).getShowText(storeText);
            case 2:
               return (new NumberCellProperty(cdp)).getShowText(storeText);
            case 3:
               return (new CurrencyCellProperty(cdp)).getShowText(storeText);
            default:
               return storeText;
         }
      }
   }

   public final void setShowText(String showText) {
      if (showText != null && showText.length() != 0) {
         this.setCellData(this.parseShowText(showText));
      } else {
         this.setCellData("");
      }

   }

   public final String parseEditText(String editText) {
      int dataType = this.getDataType();
      CellDataProperty cdp = new CellDataProperty(this.getDataType(), this.getDataFlag(), this.getEditMode(), this.getDataFormat(), this.getDataProperty());
      switch (dataType) {
         case 1:
            return (new TextCellProperty(cdp)).parseEditText(editText);
         case 2:
            return (new NumberCellProperty(cdp)).parseEditText(editText);
         case 3:
            return (new CurrencyCellProperty(cdp)).parseEditText(editText);
         default:
            return editText;
      }
   }

   public final String parseShowText(String showText) {
      int dataType = this.getDataType();
      CellDataProperty cdp = new CellDataProperty(this.getDataType(), this.getDataFlag(), this.getEditMode(), this.getDataFormat(), this.getDataProperty());
      switch (dataType) {
         case 1:
            return (new TextCellProperty(cdp)).parseShowText(showText);
         case 2:
            return (new NumberCellProperty(cdp)).parseShowText(showText);
         case 3:
            return (new CurrencyCellProperty(cdp)).parseShowText(showText);
         default:
            return showText;
      }
   }

   public boolean equals(Object obj) {
      if (this == obj) {
         return true;
      } else if (this != null && obj != null) {
         if (!(obj instanceof GridCell)) {
            return false;
         } else {
            GridCell gridCell = (GridCell)obj;
            byte[] cmpData = gridCell.getPropData();
            if (this.data.length != cmpData.length) {
               return false;
            } else {
               for(int i = 0; i < this.data.length; ++i) {
                  if (this.data[i] != cmpData[i]) {
                     return false;
                  }
               }

               int srcCellLeftBorder = this.col == 0 ? 0 : this.grid.getCellPropData(this.col - 1, this.row)[9];
               int srcCellTopBorder = this.row == 0 ? 0 : this.grid.getCellPropData(this.col, this.row - 1)[10];
               int tarCellLeftBorder = this.col == 0 ? 0 : this.grid.getCellPropData(gridCell.getColNum() - 1, gridCell.getRowNum())[9];
               int tarCellTopBorder = this.row == 0 ? 0 : this.grid.getCellPropData(gridCell.getColNum(), gridCell.getRowNum() - 1)[10];
               return srcCellLeftBorder == tarCellLeftBorder && srcCellTopBorder == tarCellTopBorder;
            }
         }
      } else {
         return false;
      }
   }

   public int hashCode() {
      int result = 0;

      for(int i = 0; i < this.data.length; ++i) {
         if (i == 9) {
            result = 37 * result + (this.col == 0 ? 0 : this.grid.getCellPropData(this.col - 1, this.row)[9]);
         } else if (i == 10) {
            result = 37 * result + (this.row == 0 ? 0 : this.grid.getCellPropData(this.col, this.row - 1)[10]);
         } else {
            result = 37 * result + this.data[i];
         }
      }

      return result;
   }

   public final NumberCellPropertyIntf toNumberCell() {
      return new NumberCellProperty(this);
   }

   public final TextCellPropertyIntf toTextCell() {
      return new TextCellProperty(this);
   }

   public final CurrencyCellPropertyIntf toCurrencyCell() {
      return new CurrencyCellProperty(this);
   }
}
